@@ -32,6 +32,8 @@
 
 #define noKBDRAW_DEBUG
 
+#define INVALID_KEYCODE ((guint)(-1))
+
 enum {
 	BAD_KEYCODE = 0,
 	NUM_SIGNALS
@@ -527,21 +529,70 @@ parse_xkb_color_spec (gchar * colorspec, GdkColor * color)
 static guint
 find_keycode (GkbdKeyboardDrawing * drawing, gchar * key_name)
 {
-	guint i;
+#define KEYSYM_NAME_MAX_LENGTH 4
+	guint keycode;
+	gint i, j;
+	XkbKeyNamePtr pkey;
+	XkbKeyAliasPtr palias;
+	guint is_name_matched;
+	gchar *src, *dst;
 
 	if (!drawing->xkb)
-		return (gint) (-1);
+		return INVALID_KEYCODE;
 
-	for (i = drawing->xkb->min_key_code;
-	     i <= drawing->xkb->max_key_code; i++) {
-		if (drawing->xkb->names->keys[i].name[0] == key_name[0]
-		    && drawing->xkb->names->keys[i].name[1] == key_name[1]
-		    && drawing->xkb->names->keys[i].name[2] == key_name[2]
-		    && drawing->xkb->names->keys[i].name[3] == key_name[3])
-			return i;
+#ifdef KBDRAW_DEBUG
+	printf ("    looking for keycode for (%c%c%c%c)\n",
+		key_name[0], key_name[1], key_name[2], key_name[3]);
+#endif
+
+	pkey = drawing->xkb->names->keys + drawing->xkb->min_key_code;
+	for (keycode = drawing->xkb->min_key_code;
+	     keycode <= drawing->xkb->max_key_code; keycode++) {
+		is_name_matched = 1;
+		src = key_name;
+		dst = pkey->name;
+		for (i = KEYSYM_NAME_MAX_LENGTH; --i >= 0;) {
+			if ('\0' == *src)
+				break;
+			if (*src++ != *dst++) {
+				is_name_matched = 0;
+				break;
+			}
+		}
+		if (is_name_matched) {
+#ifdef KBDRAW_DEBUG
+			printf ("      found keycode %u\n", keycode);
+#endif
+			return keycode;
+		}
+		pkey++;
 	}
 
-	return (guint) (-1);
+	palias = drawing->xkb->names->key_aliases;
+	for (j = drawing->xkb->names->num_key_aliases; --j >= 0;) {
+		is_name_matched = 1;
+		src = key_name;
+		dst = palias->alias;
+		for (i = KEYSYM_NAME_MAX_LENGTH; --i >= 0;) {
+			if ('\0' == *src)
+				break;
+			if (*src++ != *dst++) {
+				is_name_matched = 0;
+				break;
+			}
+		}
+
+		if (is_name_matched) {
+			keycode = find_keycode (drawing, palias->real);
+#ifdef KBDRAW_DEBUG
+			printf ("found alias keycode %u\n", keycode);
+#endif
+			return keycode;
+		}
+		palias++;
+	}
+
+	return INVALID_KEYCODE;
 }
 
 
@@ -1491,9 +1542,11 @@ init_keys_and_doodads (GkbdKeyboardDrawing * drawing)
 							      xkbkey->name.
 							      name);
 
+				if (keycode == INVALID_KEYCODE)
+					return;
 #ifdef KBDRAW_DEBUG
 				printf
-				    ("    initing key %d, shape: %p(%p + %d), code: %d\n",
+				    ("    initing key %d, shape: %p(%p + %d), code: %u\n",
 				     k, shape, drawing->xkb->geom->shapes,
 				     xkbkey->shape_ndx, keycode);
 #endif
